package jp.co.intra_mart.sample.im_workflow.tsfw.domain.service;

import java.util.Map;

import javax.inject.Inject;

import jp.co.intra_mart.common.aid.jdk.java.lang.StringUtil;
import jp.co.intra_mart.common.aid.jdk.java.util.LocaleUtil;
import jp.co.intra_mart.foundation.commons.validation.InputCheck;
import jp.co.intra_mart.foundation.security.exception.AccessSecurityException;
import jp.co.intra_mart.foundation.security.message.MessageManager;
import jp.co.intra_mart.foundation.workflow.application.general.UserActvMatterPropertyValue;
import jp.co.intra_mart.foundation.workflow.application.model.UserMatterPropertyModel;
import jp.co.intra_mart.foundation.workflow.exception.WorkflowException;
import jp.co.intra_mart.foundation.workflow.exception.WorkflowExternalException;
import jp.co.intra_mart.foundation.workflow.plugin.process.action.ActionProcessParameter;
import jp.co.intra_mart.foundation.workflow.util.WorkflowNumberingManager;
import jp.co.intra_mart.sample.im_workflow.ActionProcessService;
import jp.co.intra_mart.sample.im_workflow.tsfw.domain.model.SampleImwTPurchase;
import jp.co.intra_mart.sample.im_workflow.tsfw.domain.repository.SampleImwTPurchaseRepository;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional(propagation = Propagation.MANDATORY)
public class ActionProcessServiceImpl implements ActionProcessService {

    @Inject
    SampleImwTPurchaseRepository repository;

    private final String PURCHASE_MATTER_PROPERTY_KEY = "item_total";

    // 申請
    @Override
    public final String apply(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        System.out.println("----- ActionProcessParameter - apply -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - apply -----");

        // サーバーサイドバリデーションの実行
        validate(parameter, userParameter);

        // 案件番号の採番
        String number = null;
        try {
            number = WorkflowNumberingManager.getNumber();
        } catch (final WorkflowException e) {
            throw new WorkflowExternalException(MessageManager.getInstance().getMessage(LocaleUtil.toLocale(parameter.getLocaleId()), "SAMPLE.IMW.ERR.003"));
        }

        // 入力データのテーブルへの登録
        final SampleImwTPurchase entity = getEntity(parameter, userParameter);
        repository.insert(entity);

        // 入力データの案件プロパティへの登録
        createMatterProperty(parameter.getUserDataId(), PURCHASE_MATTER_PROPERTY_KEY, entity.getItemTotal());
        return number;
    }

    // 申請（一時保存）
    @Override
    public final String applyFromTempSave(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - applyFromTempSave -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - applyFromTempSave -----");

        // サーバーサイドバリデーションの実行
        validate(parameter, userParameter);

        // 案件番号の採番
        String number = null;
        try {
            number = WorkflowNumberingManager.getNumber();
        } catch (final WorkflowException e) {
            throw new WorkflowExternalException(MessageManager.getInstance().getMessage(LocaleUtil.toLocale(parameter.getLocaleId()), "SAMPLE.IMW.ERR.003"));
        }

        // 入力データのテーブルへの登録
        final SampleImwTPurchase entity = getEntity(parameter, userParameter);
        repository.update(entity);

        // 入力データの案件プロパティへの登録
        updateMatterProperty(parameter.getUserDataId(), PURCHASE_MATTER_PROPERTY_KEY, entity.getItemTotal());
        return number;
    }

    // 申請(未申請状態案件)
    @Override
    public final String applyFromUnapply(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        System.out.println("----- ActionProcessParameter - apply -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - apply -----");

        // サーバーサイドバリデーションの実行
        validate(parameter, userParameter);

        // 案件番号の採番
        String number = null;
        try {
            number = WorkflowNumberingManager.getNumber();
        } catch (final WorkflowException e) {
            throw new WorkflowExternalException(MessageManager.getInstance().getMessage(LocaleUtil.toLocale(parameter.getLocaleId()), "SAMPLE.IMW.ERR.003"));
        }

        // 入力データのテーブルへの登録
        final SampleImwTPurchase entity = getEntity(parameter, userParameter);
        repository.update(entity);

        // 入力データの案件プロパティへの登録
        createMatterProperty(parameter.getUserDataId(), PURCHASE_MATTER_PROPERTY_KEY, entity.getItemTotal());
        return number;
    }

    // 承認
    @Override
    public void approve(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - discontinue -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - discontinue -----");
    }

    // 承認終了
    @Override
    public final void approveEnd(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - discontinue -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - discontinue -----");
    }

    // 否認
    @Override
    public final void deny(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - discontinue -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - discontinue -----");
    }

    // 取止め
    @Override
    public final void discontinue(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - discontinue -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - discontinue -----");
    }

    // 案件操作
    @Override
    public final void matterHandle(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - matterHandle -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - matterHandle -----");
    }

    // 引戻し
    @Override
    public final void pullBack(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - discontinue -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - discontinue -----");
    }

    // 再申請
    @Override
    public final String reapply(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - reapply -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - reapply -----");

        // サーバーサイドバリデーションの実行
        validate(parameter, userParameter);

        // 入力データのテーブルへの登録
        final SampleImwTPurchase entity = getEntity(parameter, userParameter);
        repository.update(entity);

        // 入力データの案件プロパティへの登録
        updateMatterProperty(parameter.getUserDataId(), PURCHASE_MATTER_PROPERTY_KEY, entity.getItemTotal());
        return null;
    }

    // 保留
    @Override
    public final void reserve(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - discontinue -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - discontinue -----");
    }

    // 保留解除
    @Override
    public final void reserveCancel(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - discontinue -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - discontinue -----");
    }

    // 差戻し
    @Override
    public final void sendBack(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - discontinue -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - discontinue -----");
    }

    // 差戻し後引戻し
    @Override
    public final void sendBackToPullBack(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - discontinue -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - discontinue -----");
    }

    // 一時保存(新規登録)
    @Override
    public final void tempSaveCreate(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - tempSaveCreate -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - tempSaveCreate -----");

        // 入力データのテーブルへの登録
        final SampleImwTPurchase entity = getEntity(parameter, userParameter);
        repository.insert(entity);

        // 入力データの案件プロパティへの登録
        createMatterProperty(parameter.getUserDataId(), PURCHASE_MATTER_PROPERTY_KEY, entity.getItemTotal());

    }

    // 一時保存(削除)
    @Override
    public final void tempSaveDelete(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - tempSaveDelete -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - tempSaveDelete -----");

        // 入力データのレコードを削除
        repository.delete(parameter.getUserDataId());

        // 案件プロパティの削除
        deleteMatterProperty(parameter.getUserDataId(), PURCHASE_MATTER_PROPERTY_KEY);

    }

    // 一時保存(更新)
    @Override
    public final void tempSaveUpdate(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws Exception {
        // パラメータのログ出力
        System.out.println("----- ActionProcessParameter - tempSaveUpdate -----");
        outputLog(parameter);
        System.out.println("----- ActionProcessParameter - tempSaveUpdate -----");

        // 入力データのテーブルへの更新
        final SampleImwTPurchase entity = getEntity(parameter, userParameter);
        repository.update(entity);

        // 入力データの案件プロパティへの登録
        updateMatterProperty(parameter.getUserDataId(), PURCHASE_MATTER_PROPERTY_KEY, entity.getItemTotal());

    }

    /**
     * 指定された案件に紐づく案件プロパティの登録処理を行います。<BR>
     * <BR>
     * @param userDataId sample_imw_t_purchase ユーザー登録データID
     * @param matterPropertyKey 案件プロパティキー
     * @param matterPropertyValue 案件プロパティ値
     * @throws WorkflowException
     */
    private final void createMatterProperty(final String userDataId, final String matterPropertyKey, final String matterPropertyValue) throws WorkflowException {
        final UserMatterPropertyModel matterPropertyModel = new UserMatterPropertyModel();
        matterPropertyModel.setUserDataId(userDataId);
        matterPropertyModel.setMatterPropertyKey(matterPropertyKey);
        matterPropertyModel.setMatterPropertyValue(matterPropertyValue);

        UserActvMatterPropertyValue property;
        property = new UserActvMatterPropertyValue();
        final UserMatterPropertyModel[] matterProperty = new UserMatterPropertyModel[1];
        matterProperty[0] = matterPropertyModel;
        property.createMatterProperty(matterProperty);
    }

    /**
     * 指定された案件に紐づく案件プロパティの削除処理を行います。<BR>
     * <BR>
     * @param userDataId sample_imw_t_purchase ユーザー登録データID
     * @param matterPropertyKey 案件プロパティキー
     * @throws WorkflowException
     */
    private final void deleteMatterProperty(final String userDataId, final String matterPropertyKey) throws WorkflowException {
        final UserMatterPropertyModel matterPropertyModel = new UserMatterPropertyModel();
        matterPropertyModel.setUserDataId(userDataId);
        matterPropertyModel.setMatterPropertyKey(matterPropertyKey);

        UserActvMatterPropertyValue property;
        property = new UserActvMatterPropertyValue();
        final UserMatterPropertyModel[] matterProperty = new UserMatterPropertyModel[1];
        matterProperty[0] = matterPropertyModel;
        property.deleteMatterProperty(matterProperty);
    }

    private SampleImwTPurchase getEntity(final ActionProcessParameter parameter, final Map<String, Object> userParameter) {
        final SampleImwTPurchase entity = new SampleImwTPurchase();
        final String itemTotal = Integer.valueOf((String) userParameter.get("item_amount")).intValue() * Integer.valueOf((String) userParameter.get("item_price")).intValue() + "";

        entity.setUserDataId(parameter.getUserDataId());
        entity.setSystemMatterId(parameter.getSystemMatterId());
        entity.setItemName((String) userParameter.get("item_name"));
        entity.setItemAmount((String) userParameter.get("item_amount"));
        entity.setItemPrice((String) userParameter.get("item_price"));
        entity.setItemTotal(itemTotal);
        entity.setItemComment((String) userParameter.get("item_comment"));
        entity.setEndFlag("0");
        entity.setArchiveFlag("0");
        return entity;
    }

    private void outputLog(final ActionProcessParameter parameter) {
        System.out.println("LoginGroupId        : " + parameter.getLoginGroupId());
        System.out.println("LocaleId            : " + parameter.getLocaleId());
        if (parameter.getTargetLocales() != null) {
            System.out.print("TargetLocales       : ");
            for (final String str : parameter.getTargetLocales()) {
                System.out.print(str + " ");
            }
            System.out.println();
        } else {
            System.out.println("TargetLocales       : ");
        }
        System.out.println("ContentsId          : " + parameter.getContentsId());
        System.out.println("ContentsVersionId   : " + parameter.getContentsVersionId());
        System.out.println("RouteId             : " + parameter.getRouteId());
        System.out.println("RouteVersionId      : " + parameter.getRouteVersionId());
        System.out.println("FlowId              : " + parameter.getFlowId());
        System.out.println("FlowVersionId       : " + parameter.getFlowVersionId());
        System.out.println("ApplyBaseDate       : " + parameter.getApplyBaseDate());
        System.out.println("ProcessDate         : " + parameter.getProcessDate());
        System.out.println("SystemMatterId      : " + parameter.getSystemMatterId());
        System.out.println("UserDataId          : " + parameter.getUserDataId());
        System.out.println("MatterName          : " + parameter.getMatterName());
        System.out.println("MatterNumber        : " + parameter.getMatterNumber());
        System.out.println("PriorityLevel       : " + parameter.getPriorityLevel());
        System.out.println("Parameter           : " + parameter.getParameter());
        System.out.println("ActFlag             : " + parameter.getActFlag());
        System.out.println("NodeId              : " + parameter.getNodeId());
        if (parameter.getNextNodeIds() != null) {
            System.out.print("NextNodeIds         : ");
            for (final String str : parameter.getNextNodeIds()) {
                System.out.print(str + " ");
            }
            System.out.println();
        } else {
            System.out.println("NextNodeIds         : ");
        }
        System.out.println("AuthUserCd          : " + parameter.getAuthUserCd());
        System.out.println("ExecUserCd          : " + parameter.getExecUserCd());
        System.out.println("ResultStatus        : " + parameter.getResultStatus());
        System.out.println("AuthCompanyCode     : " + parameter.getAuthCompanyCode());
        System.out.println("AuthOrgzSetCode     : " + parameter.getAuthOrgzSetCode());
        System.out.println("AuthOrgzCode        : " + parameter.getAuthOrgzCode());
        System.out.println("ProcessComment      : " + parameter.getProcessComment());
        System.out.println("LumpProcessFlag     : " + parameter.getLumpProcessFlag());
        System.out.println("AutoProcessFlag     : " + parameter.getAutoProcessFlag());
    }

    /**
     * 指定された案件に紐づく案件プロパティの更新処理を行います。<BR>
     * <BR>
     * @param userDataId sample_imw_t_purchase ユーザー登録データID
     * @param matterPropertyKey 案件プロパティキー
     * @param matterPropertyValue 案件プロパティ値
     * @throws WorkflowException
     */
    private final void updateMatterProperty(final String userDataId, final String matterPropertyKey, final String matterPropertyValue) throws WorkflowException {
        final UserMatterPropertyModel matterPropertyModel = new UserMatterPropertyModel();
        matterPropertyModel.setUserDataId(userDataId);
        matterPropertyModel.setMatterPropertyKey(matterPropertyKey);
        matterPropertyModel.setMatterPropertyValue(matterPropertyValue);

        UserActvMatterPropertyValue property;
        property = new UserActvMatterPropertyValue();
        final UserMatterPropertyModel[] matterProperty = new UserMatterPropertyModel[1];
        matterProperty[0] = matterPropertyModel;
        property.updateMatterProperty(matterProperty);
    }

    // サーバーサイドバリデーションのメソッド
    private void validate(final ActionProcessParameter parameter, final Map<String, Object> userParameter) throws WorkflowException, AccessSecurityException {
        if (StringUtil.isBlank((String) userParameter.get("item_name"))) {
            throw new WorkflowExternalException(MessageManager.getInstance().getMessage(LocaleUtil.toLocale(parameter.getLocaleId()), "SAMPLE.IMW.INF.000"));
        }

        if (StringUtil.isBlank((String) userParameter.get("item_amount"))) {
            throw new WorkflowExternalException(MessageManager.getInstance().getMessage(LocaleUtil.toLocale(parameter.getLocaleId()), "SAMPLE.IMW.INF.001"));
        }

        if (StringUtil.isBlank((String) userParameter.get("item_price"))) {
            throw new WorkflowExternalException(MessageManager.getInstance().getMessage(LocaleUtil.toLocale(parameter.getLocaleId()), "SAMPLE.IMW.INF.002"));
        }

        if (!InputCheck.isNumberString((String) userParameter.get("item_amount"))) {
            throw new WorkflowExternalException(MessageManager.getInstance().getMessage(LocaleUtil.toLocale(parameter.getLocaleId()), "SAMPLE.IMW.INF.003"));
        }

        if (!InputCheck.isNumberString((String) userParameter.get("item_price"))) {
            throw new WorkflowExternalException(MessageManager.getInstance().getMessage(LocaleUtil.toLocale(parameter.getLocaleId()), "SAMPLE.IMW.INF.004"));
        }
    }
}
